const masterTimeline = gsap.timeline({ repeat: -1 });

function eyeBlink() {
  const anim = gsap
    .timeline({
      repeat: -1,
      repeatDelay: 1.5,
      defaults: { ease: "circ", duration: 0.1 },
    })
    .to("#eyes-left, #eyes-right", { scaleY: 0 })
    .to("#eyes-left, #eyes-right", { scaleY: 1 });
  return anim;
}

function sleep() {
  gsap.set("#eyes-left, #eyes-right", {
    opacity: 0,
  });

  gsap.set("#eye-shut-right, #eye-shut-left, #sleep-zz", {
    opacity: 1,
  });

  const anim = gsap
    .timeline({
      repeat: -1,
      defaults: {
        duration: 1,
      },
    })
    .to("#sleep-zz *", {
      y: -10,
      opacity: 0,
      delay: 0.5,
      stagger: 0.3,
    });
  return anim;
}

function breathNormal() {
  const anim = gsap
    .timeline({
      repeat: -1,
      repeatDelay: 2,
    })
    .to("#mouth", { scaleY: 3, scaleX: 0.4 })
    .to("#mouth", { scaleY: 1, scaleX: 1 });
  return anim;
}

function Idle() {
  const anim = gsap
    .timeline({
      repeat: -1,
      repeatDelay: 0.2,
    })
    .to("#char", { y: 5 })
    .to("#char", { y: 0 });
  return anim;
}

function init() {
  gsap.set("#eyes-left, #eyes-right, #mouth", {
    transformOrigin: "50% 100%",
    scaleY: 0,
  });
  gsap.set(
    "#eye-shut-right, #eye-shut-left, #headband, #sweat-drops, #lift-barbell, #sleep-zz",
    {
      opacity: 0,
    }
  );
  const container = document.querySelector(".character-img");
  container.style.display = "block";
}

function closedEyes() {
  gsap.set("#eyes-left, #eyes-right", {
    opacity: 0,
  });

  gsap.set("#eye-shut-right, #eye-shut-left, #sleep-zz", {
    opacity: 1,
  });
}

function openEyes() {
  gsap.set("#eyes-left, #eyes-right", {
    opacity: 1,
  });

  gsap.set("#eye-shut-right, #eye-shut-left, #sleep-zz", {
    opacity: 0,
  });
}

document.addEventListener("DOMContentLoaded", () => {
  const container = document.querySelector(".animationContainer");

  const mouseCoords = container.getBoundingClientRect();
  container.addEventListener("mousemove", (e) => {
    const mouseX = e.pageX - container.offsetLeft;
    const mouseY = e.pageY - container.offsetTop;
    gsap.to("#eye-black-right, #eye-black-left", 0.3, {
      x: ((mouseX - mouseCoords.width / 2) / mouseCoords.width) * 7,
      y: ((mouseY - mouseCoords.height / 2) / mouseCoords.width) * 20,
      ease: "power3.out",
    });
  });

  container.addEventListener("mouseleave", (e) => {
    gsap.to("#eye-black-right, #eye-black-left", 0.3, {
      x: 0,
      y: 0,
    });
  });

  init();

  const idleAnimation = gsap.timeline({ paused: true });
  idleAnimation.add(eyeBlink);
  idleAnimation.add(breathNormal);

  const sleepAnimation = gsap.timeline({ paused: true });
  sleepAnimation.add(sleep);
  sleepAnimation.add(breathNormal);

  const workoutAnimation = gsap
    .timeline({
      repeat: -1,
      paused: true,
      defaults: {
        duration: 1,
      },
    })
    .to("#sweat-drops *", {
      y: 10,
      opacity: 0,
      delay: 0.5,
      stagger: 0.3,
    })
    .to("#lift-barbell", {
      y: -30,
    })
    .to("#lift-barbell", {
      y: 0,
    });

  chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    // update the timer progress bar
    if (message.action === "startAnimation") {
      idleAnimation.pause();
      workoutAnimation.pause();
      sleepAnimation.pause();

      if (message.animationName === "workout") {
        openEyes();
        gsap.set("#headband, #sweat-drops, #lift-barbell", {
          opacity: 1,
        });

        idleAnimation.play();
        workoutAnimation.play();
      }

      if (message.animationName === "sleep") {
        closedEyes();
        gsap.set("#headband, #sweat-drops, #lift-barbell", {
          opacity: 0,
        });
        sleepAnimation.play();
      }

      if (message.animationName === "idle") {
        openEyes();
        gsap.set("#headband, #sweat-drops, #lift-barbell", {
          opacity: 0,
        });
        idleAnimation.play();
      }
    }
  });
});
