const timeOptions = [
  { text: "1 min", value: 1 },
  { text: "2 mins", value: 2 },
  { text: "5 mins", value: 5 },
  { text: "10 mins", value: 10 },
  { text: "15 mins", value: 15 },
];

const renderDropdown = (optionValues, dropdownElement) => {
  optionValues.map((option) => {
    let optionElement = document.createElement("option");
    optionElement.text = option.text;
    optionElement.value = option.value;
    dropdownElement.append(optionElement);
  });
};

document.addEventListener("DOMContentLoaded", () => {
  const autoBlocklistContainer = document.querySelector(
    ".autoBlocklistContainer"
  );

  const autoblockBlockButton = document.querySelector(".autoblockBlockButton");
  const autoBlockSelectedTime = document.querySelector(".autoBlockTimeOption");
  const autoBlockSelectedSite = document.querySelector(".autoBlockSiteOption");

  const autoBlockToggle = document.querySelector("#autoblockToggle");
  const hardModeToggle = document.querySelector("#hardModeToggle");
  const soundToggle = document.querySelector("#soundToggle");
  const notifiationToggle = document.querySelector("#notifiationToggle");
  const breakModeToggle = document.querySelector("#breakModeToggle");

  breakModeToggle.addEventListener("click", (event) => {
    chrome.runtime.sendMessage({
      action: "breakModeToggle",
    });
  });

  notifiationToggle.addEventListener("click", (event) => {
    chrome.runtime.sendMessage({
      action: "notificationToggle",
    });
  });

  soundToggle.addEventListener("click", (event) => {
    chrome.runtime.sendMessage({
      action: "soundToggle",
    });
  });

  autoBlockToggle.addEventListener("click", (event) => {
    chrome.runtime.sendMessage({
      action: "toggleAutoBlock",
    });
  });

  hardModeToggle.addEventListener("click", (event) => {
    chrome.runtime.sendMessage({
      action: "toggleHardMode",
    });
  });

  soundToggle.addEventListener("click", () => trackButton("sound_Toggle"));

  autoBlockToggle.addEventListener("click", () =>
    trackButton("autoblock_toggle")
  );

  hardModeToggle.addEventListener("click", () =>
    trackButton("hardmode_toggle")
  );

  autoblockBlockButton.addEventListener("click", () =>
    trackButton("add_site_to_autoblock")
  );

  breakModeToggle.addEventListener("click", () =>
    trackButton("breakmode_toggle")
  );

  // update UI with initial state
  chrome.runtime.sendMessage(actions.getAllState, function (response) {
    // set autoblock state
    const autoblockState = response.payload.autoblock;
    autoBlockToggle.checked = autoblockState;

    const soundsOnState = response.payload.soundsOn;
    soundToggle.checked = soundsOnState;

    const hardModeState = response.payload.hardMode;
    hardModeToggle.checked = hardModeState;

    const notificationState = response.payload.pushNotificationsEnabled;
    notifiationToggle.checked = notificationState;

    const breakModeState = response.payload.disableBlockInBreakMode;
    breakModeToggle.checked = breakModeState;
  });

  const renderList = () => {
    chrome.runtime.sendMessage(actions.getAllState, function (response) {
      const autoBlockedSites = response.payload.autoBlockedSites;
      if (autoBlockedSites.length <= 0) {
        // show place holder image
        autoblockBlockButton.classList.remove("buttonSuccess");
        autoblockBlockButton.innerHTML = "Block";

        const placeholderImage = document.createElement("img");
        placeholderImage.src = "/images/mountainIcon.svg";

        const text = document.createElement("p");
        text.classList.add("autoblockPlaceholderLabel");
        text.textContent =
          "Automatically block sites after you spend a certain duration on them";

        autoBlocklistContainer.appendChild(placeholderImage);
        autoBlocklistContainer.appendChild(text);
      } else {
        autoBlockedSites.map((site) => {
          renderAutoBlockedList(site);
        });
      }
    });
  };

  renderDropdown(timeOptions, autoBlockSelectedTime);
  renderWebsiteDropdown();
  renderList();

  function renderWebsiteDropdown() {
    chrome.runtime.sendMessage(actions.getAllState, function (response) {
      const blockedSites = response.payload.blockedSites;
      const blockedSiteList = [];
      blockedSites.map((x) => blockedSiteList.push({ text: x, value: x }));
      renderDropdown(blockedSiteList, autoBlockSelectedSite);
    });
  }

  const addSiteToAutoblockList = (event) => {
    if (autoBlockSelectedSite.value === "new") {
      console.log("New site");
    } else {
      chrome.runtime.sendMessage(actions.getAllState, function (response) {
        const { isUserPro, autoBlockedSites } = response.payload;
        const paywallMessage = document.querySelector(".autoBlockPaywall");

        // free user
        if (!isUserPro) {
          if (autoBlockedSites.length >= 1) {
            paywallMessage.style.display = "block";
          } else {
            paywallMessage.style.display = "none";
            chrome.runtime.sendMessage(
              {
                action: "addSiteToAutoblock",
                payload: {
                  url: autoBlockSelectedSite.value,
                  time: autoBlockSelectedTime.value,
                },
              },
              (response) => {
                const button = event.target;
                button.innerHTML = "Blocked";
                button.classList.add("buttonSuccess");
                clearChildrenFor(autoBlocklistContainer);
                renderList();
              }
            );
          }
        }

        // pro user
        if (isUserPro) {
          paywallMessage.style.display = "none";
          chrome.runtime.sendMessage(
            {
              action: "addSiteToAutoblock",
              payload: {
                url: autoBlockSelectedSite.value,
                time: autoBlockSelectedTime.value,
              },
            },
            (response) => {
              const button = event.target;
              button.innerHTML = "Blocked";
              button.classList.add("buttonSuccess");
              clearChildrenFor(autoBlocklistContainer);
              renderList();
            }
          );
        }
      });
    }
  };

  const clearChildrenFor = (parentNode) => {
    while (parentNode.firstChild) {
      parentNode.removeChild(parentNode.lastChild);
    }
  };

  const renderAutoBlockedList = (site) => {
    // wrapper
    let element = document.createElement("div");
    element.classList.add("autoBlockItem");
    // site label
    let span = document.createElement("span");
    span.classList.add("autoBlockName");
    span.innerHTML = site.siteName;

    // time pill label
    let timeLabel = document.createElement("span");
    timeLabel.classList.add("autoBlockTextTime");
    timeLabel.innerHTML = parseInt(site.duration) / 60 + " min";

    // block button
    let button = document.createElement("button");
    button.classList.add("autoBlockUnblock");
    button.innerHTML = site.blocked ? "unblock" : "";
    button.dataset.url = site.siteName;
    button.addEventListener("click", removeAutoBlockURLFromList);

    // delete button
    let deleteButton = document.createElement("button");
    deleteButton.classList.add("autoBlockDelete");
    deleteButton.dataset.url = site.siteName;

    // delete button icon
    let deleteImage = document.createElement("img");
    deleteImage.src = "/images/x.svg";
    deleteImage.dataset.url = site.siteName;
    deleteButton.append(deleteImage);
    deleteButton.dataset.url = site.siteName;
    deleteButton.addEventListener("click", deleteURLFromList);

    element.appendChild(span);
    element.appendChild(timeLabel);
    site.blocked && element.appendChild(button); // only render the unblock button if the site is blocked
    element.appendChild(deleteButton);

    autoBlocklistContainer.appendChild(element);
  };

  const removeAutoBlockURLFromList = (event) => {
    const url = event.target.dataset.url;
    chrome.runtime.sendMessage(
      { action: "requestAutoBlockUrlOff", payload: { url: url } },
      (response) => {
        clearChildrenFor(autoBlocklistContainer);
        renderList();
      }
    );
  };

  const deleteURLFromList = (event) => {
    const url = event.target.dataset.url;
    chrome.runtime.sendMessage(
      { action: "requestAutoBlockDeleteURL", payload: { url: url } },
      (response) => {
        const paywallMessage = document.querySelector(".autoBlockPaywall");
        paywallMessage.style.display = "none";
        clearChildrenFor(autoBlocklistContainer);
        renderList();
      }
    );
  };

  const resetBlockButton = () => {
    autoblockBlockButton.innerHTML = "Block";
    autoblockBlockButton.classList.remove("buttonSuccess");
  };

  autoblockBlockButton.addEventListener("click", addSiteToAutoblockList);
  autoBlockSelectedTime.addEventListener("change", resetBlockButton);
  autoBlockSelectedSite.addEventListener("change", resetBlockButton);
});
