function percentage(partialValue, totalValue) {
  return (100 * partialValue) / totalValue;
}

function generateHSLPalatte(count) {
  let light = 50;
  let cl = [];

  for (let i = 0; i < count; i++) {
    let n = light + percentage(i * 5, 100);
    cl.push(`hsl(227, 100%, ${n}%)`);
  }
  return cl;
}

const sortedValues = (data) => {
  return Object.entries(data)
    .sort((a, b) => b[1] - a[1])
    .reduce(
      (_sortedObj, [k, v]) => ({
        ..._sortedObj,
        [k]: v,
      }),
      {}
    );
};

function getTopDurationsData(data) {
  const sortedDecending = sortedValues(data);
  const sliced = Object.keys(sortedDecending)
    .slice(0, 8)
    .reduce((result, key) => {
      result[key] = sortedDecending[key];

      return result;
    }, {});

  return sliced;
}

let getDaysArray = function (start, end) {
  for (
    var arr = [], dt = new Date(start);
    dt <= end;
    dt.setDate(dt.getDate() + 1)
  ) {
    arr.push(new Date(dt));
  }
  return arr;
};

function generateDayFormat(date) {
  return date
    .toLocaleDateString("en-US", {
      day: "numeric",
      month: "short",
      year: "numeric",
    })
    .replace(/ /g, "/")
    .replace(",", "");
}

function formatWeeklySiteDataObject(weeklyData, daysInBetweenFormatted) {
  let sitesFormattedByWeek = {};
  Object.keys(weeklyData).forEach(function (sitename) {
    let formattedDays = Object.keys(weeklyData[sitename]);
    let o = {};
    daysInBetweenFormatted.map((date) => {
      if (formattedDays.includes(date)) {
        let dayName = new Date(date).toLocaleString("en-us", {
          weekday: "short",
        });
        o[dayName] = Math.floor(weeklyData[sitename][date]?.seconds / 60);
      } else {
        let dayName = new Date(date).toLocaleString("en-us", {
          weekday: "short",
        });
        o[dayName] = 0;
      }
    });
    sitesFormattedByWeek[sitename] = o;
  });
  return sitesFormattedByWeek;
}

function renderAdvancedInsights(result) {
  const weeklyData = result.weeklyInsights;
  const timeSpentData = getTopDurationsData(result.topInsightsData);
  const totalTimeInSec = Object.values(timeSpentData).reduce(
    (a, b) => a + b,
    0
  );

  const keys = Object.keys(timeSpentData);
  const values = Object.values(timeSpentData).map((x) =>
    percentage(x, totalTimeInSec).toFixed(2)
  );

  let startOfWeek = dayjs().startOf("week");
  let endOfWeek = dayjs().endOf("week");
  let daysInBetween = getDaysArray(startOfWeek, endOfWeek);
  let daysInBetweenFormatted = daysInBetween.map((x) => generateDayFormat(x));

  let sitesFormattedByWeek = formatWeeklySiteDataObject(
    weeklyData,
    daysInBetweenFormatted
  );

  const weeklyDataContainer = document.querySelector(
    ".advanceWeeklyGraphContainer"
  );

  const weeklyDataContainerHeader = document.querySelector(
    ".advanceContainerHeader"
  );

  let siteDropDown = document.createElement("select");
  siteDropDown.classList.add("dropdownSites");
  Object.keys(sitesFormattedByWeek).forEach(function (site) {
    let dropdown = document.createElement("option");
    dropdown.innerHTML = site;
    dropdown.value = site;
    siteDropDown.appendChild(dropdown);
  });

  siteDropDown.addEventListener("change", (event) => {
    let siteName = event.target.value;
    const currentSelectedSiteData = sitesFormattedByWeek[siteName];
    const currentSelectedSiteDataKeys = Object.keys(currentSelectedSiteData);
    const currentSelectedSiteDataValues = Object.values(
      currentSelectedSiteData
    );

    chart.data.datasets.pop();
    chart.data.datasets.push({
      backgroundColor: generateHSLPalatte(currentSelectedSiteDataKeys.length),
      data: currentSelectedSiteDataValues,
    });
    chart.update();
  });

  let canvas = document.createElement("canvas");
  canvas.classList.add("graphCanvas");
  weeklyDataContainerHeader.appendChild(siteDropDown);
  weeklyDataContainer.appendChild(canvas);

  let ctx = canvas.getContext("2d");

  const currentSelectedSiteData =
    sitesFormattedByWeek[Object.keys(sitesFormattedByWeek)[0]];
  const currentSelectedSiteDataKeys = Object.keys(currentSelectedSiteData);
  const currentSelectedSiteDataValues = Object.values(currentSelectedSiteData);

  let chart;

  chrome.runtime.sendMessage(actions.getAllState, function (response) {
    const { isUserPro } = response.payload;

    if (isUserPro) {
      // pie chart
      renderChart(keys, values);
      chart = drawBarChart(
        ctx,
        currentSelectedSiteDataKeys,
        currentSelectedSiteDataValues,
        generateHSLPalatte(currentSelectedSiteDataKeys.length),
        true,
        true
      );
    } else {
      let url = "https://ottoapp.me";
      const button = document.createElement("a");
      button.textContent = "Upgrade to Pro";
      button.classList.add("upgradeButton");
      button.href = url;
      button.target = "_blank";

      const button2 = document.createElement("a");
      button2.textContent = "Upgrade to Pro";
      button2.classList.add("upgradeButton");
      button2.href = url;
      button2.target = "_blank";

      const blurContainer = document.createElement("div");
      blurContainer.classList.add("blurContainer");
      blurContainer.appendChild(button);

      const blurContainer2 = document.createElement("div");
      blurContainer2.classList.add("blurContainer");
      blurContainer2.appendChild(button2);

      const graphContainer = document.querySelector(".advanceGraph");
      weeklyDataContainer.appendChild(blurContainer);
      graphContainer.appendChild(blurContainer2);
      siteDropDown.disabled = true;
      renderChart(["x", "x", "x", "x", "x", "x"], [10, 5, 10, 5, 10, 5]);
      drawBarChart(
        ctx,
        ["x", "x", "x", "x", "x", "x"],
        [10, 5, 10, 5, 10, 5],
        generateHSLPalatte(currentSelectedSiteDataKeys.length),
        true,
        true
      );
    }
  });
}

document.addEventListener("DOMContentLoaded", () => {
  // advanced insights
  chrome.storage.local.get(["topInsightsData", "weeklyInsights"], (result) => {
    if (
      result.topInsightsData !== undefined &&
      result.weeklyInsights !== undefined
    ) {
      renderAdvancedInsights(result);
    } else {
      const weeklyDataContainer = document.querySelector(
        ".advanceWeeklyGraphContainer"
      );
      const graphContainer = document.querySelector(".advanceGraph");
      let label = "No data tracked yet";

      const div = document.createElement("div");
      div.classList.add("noDataContainer");

      const div1 = document.createElement("div");
      div1.classList.add("noDataContainer");

      const p = document.createElement("p");
      p.classList.add("noDataText");
      p.innerHTML = label;

      const p1 = document.createElement("p");
      p1.classList.add("noDataText");
      p1.innerHTML = label;

      div.appendChild(p);
      div1.appendChild(p1);

      graphContainer.appendChild(div);
      weeklyDataContainer.appendChild(div1);
    }
  });

  // basic insights
  chrome.runtime.sendMessage(actions.getAllState, function (response) {
    const payload = response.payload;

    if (payload.hasOwnProperty("insightsData")) {
      const insights = payload.insightsData;
      // ensure to create properties if they don't exist
      const dataPropertyKeys = [
        "regenerationCount",
        "totalWorkTime",
        "avgWorkHours",
        "distractionCount",
        "dailyDistractionList",
      ];
      dataPropertyKeys.map((k) => {
        if (!insights.hasOwnProperty(k)) {
          if (k === "dailyDistractionList") {
            insights[k] = {};
          } else {
            insights[k] = 0;
          }
        }
      });
      // update insights object
      chrome.runtime.sendMessage(
        {
          action: "updateInsights",
          payload: { insights: insights },
        },
        (response) => {
          if (response.status === "OK") {
            const insightsKeys = Object.keys(insights);
            const despData = {
              regenerationCount: { title: "Death Count" },
              distractionCount: { title: "Distration Count" },
              totalWorkTime: { title: "Time spent focused" },
              avgWorkHours: { title: "Hard Mode" },
            };

            insightsKeys.map((ik) => {
              if (typeof insights[ik] === "number") {
                // update DOM elements with data

                const title = despData[ik].title;
                let value = insights[ik];

                if (ik === "distractionCount") {
                  let count = Object.values(
                    insights.dailyDistractionList
                  ).reduce((a, b) => a + b, 0);
                  value = count;
                }

                if (ik === "totalWorkTime") {
                  // if minutes are greater than 60, show hours, else show mins
                  let time =
                    value > 3600
                      ? Math.floor(value / 3600)
                      : Math.floor(value / 60);
                  let timeType = value > 3600 ? "hrs" : "min";
                  value = `${time}${timeType}`;
                }
                renderData(title, value);
              }
            });
          }
        }
      );

      // only render graph if ther
      if (Object.keys(insights.dailyDistractionList).length > 0) {
        renderGraph();
      } else {
        const graphContainer = document.querySelector(".dataGraph");

        let div = document.createElement("div");
        div.classList.add("insightsEmptyContainer");

        let container = document.createElement("div");
        container.classList.add("insightDataContainerWrapper");

        let textData = document.createElement("p");
        textData.textContent = "Nothing to show yet";
        textData.classList.add("insightsEmptyText");

        let graphImg = document.createElement("img");
        graphImg.src = "/images/graphInsights.svg";

        container.appendChild(graphImg);
        container.appendChild(textData);
        div.appendChild(container);
        graphContainer.append(div);
      }
    }
  });
});

function renderData(title, value) {
  const dataContainer = document.querySelector(".dataItemContainer");

  let div = document.createElement("div");
  div.classList.add("insightDataContainer");

  let subHeading = document.createElement("p");
  subHeading.textContent = title;
  subHeading.classList.add("insightsSubheading");

  let textData = document.createElement("p");
  textData.textContent = value;
  textData.classList.add("insightsData");

  div.appendChild(subHeading);
  div.appendChild(textData);

  dataContainer.appendChild(div);
}

function renderGraph() {
  const graphContainer = document.querySelector(".dataGraph");
  let canvas = document.createElement("canvas");
  canvas.classList.add("graphCanvas");
  graphContainer.appendChild(canvas);

  let ctx = canvas.getContext("2d");
  chrome.runtime.sendMessage(actions.getAllState, function (response) {
    const payload = response.payload;
    const dataPoints = payload.insightsData.dailyDistractionList;

    drawBarChart(ctx, Object.keys(dataPoints), Object.values(dataPoints), [
      "#FFA5E0",
    ]);
  });
}

function drawBarChart(
  ctx,
  labels,
  values,
  colors,
  showGridLines = true,
  showLabel = false
) {
  let myChart = new Chart(ctx, {
    type: "bar",
    hover: false,

    data: {
      labels: labels,
      datasets: [
        {
          label: "",
          data: values,
          fill: true,
          backgroundColor: colors,
          borderWidth: 0,
          categoryPercentage: 1.0,
          barPercentage: 1.0,
        },
      ],
    },
    options: {
      weight: 0,
      legend: {
        display: false,
      },
      hover: false,
      scales: {
        xAxes: [
          {
            ticks: {
              display: showLabel, //this will remove only the label
            },
            gridLines: {
              display: showGridLines,
            },
          },
        ],
        yAxes: [
          {
            ticks: {
              beginAtZero: true,
              display: showLabel, //this will remove only the label
            },
            gridLines: {
              display: showGridLines,
            },
          },
        ],
      },
    },
  });

  return myChart;
}

function renderChart(keys, values) {
  const graphContainer = document.querySelector(".advanceGraph");
  let canvas = document.createElement("canvas");
  canvas.classList.add("graphCanvas");
  graphContainer.appendChild(canvas);

  let ctx = canvas.getContext("2d");
  chrome.runtime.sendMessage(actions.getAllState, function (response) {
    let myChart = new Chart(ctx, {
      type: "doughnut",
      hover: false,

      data: {
        labels: keys,
        datasets: [
          {
            label: "",
            data: values,
            fill: true,
            backgroundColor: generateHSLPalatte(keys.length),
            borderWidth: 0,
          },
        ],
      },
      options: {
        legend: {
          display: false,
        },
        hover: true,
        scales: {},
      },
    });
  });
}
