function stripurl(url) {
  return url !== undefined
    ? url.replace(/^(?:https?:\/\/)?(?:www\.)?/i, "").split("/")[0]
    : "";
}

class ActionBlocks {
  constructor(message, sender, sendResponse) {
    this.message = message;
    this.sendResponse = sendResponse;
    this.sender = sender;

    this.actionObject = {
      switchScreen: this.switchScreen.bind(this),
      updateTimer: this.updateTimer.bind(this),
      startTimer: this.startTimer.bind(this),
      stopTimer: this.stopTimer.bind(this),
      getTimerState: this.getTimerState.bind(this),
      siteVisited: this.siteVisited.bind(this),
      pauseTimer: this.pauseTimer.bind(this),
      playTimer: this.playTimer.bind(this),
      addSiteToBlock: this.addSiteToBlock.bind(this),
      setCountTimer: this.setCountTimer.bind(this),
      setBreakTimer: this.setBreakTimer.bind(this),
      requestBlockList: this.requestBlockList.bind(this),
      requestUnblockURL: this.requestUnblockURL.bind(this),
      requestAutoBlockUrlOff: this.requestAutoBlockUrlOff.bind(this),
      requestAutoBlockDeleteURL: this.requestAutoBlockDeleteURL.bind(this),
      startBreakTimer: this.startBreakTimer.bind(this),
      getAllState: this.getAllState.bind(this),
      depleteHealth: this.depleteHealth.bind(this),
      startAnimation: this.startAnimation.bind(this),
      redirectToBlockPage: this.redirectToBlockPage.bind(this),
      autoBlockCheck: this.autoBlockCheck.bind(this),
      addSiteToAutoblock: this.addSiteToAutoblock.bind(this),
      toggleAutoBlock: this.toggleAutoBlock.bind(this),
      toggleHardMode: this.toggleHardMode.bind(this),
      updateInsights: this.updateInsights.bind(this),
      userDistracted: this.userDistracted.bind(this),
      soundToggle: this.soundToggle.bind(this),
      notificationToggle: this.notificationToggle.bind(this),
      breakModeToggle: this.breakModeToggle.bind(this),
      activatePro: this.activatePro.bind(this),
      deactivatePro: this.deactivatePro.bind(this),
    };
  }

  // every time site is blocked an event is triggered 'userDistracted'
  // in the insights.dailyDistractionList object a key value pair is added
  // current date will be the key and the distraction count will be the value
  // if current date already exists, then it updates the count
  // if it doesn't exist, it creates one.

  deactivatePro() {}

  activatePro() {
    // let isUserPro = state.getState("isUserPro");
    // let proLicenseKey = state.getState("proLicenseKey");

    state.setState("isUserPro", true);
    state.setState("proLicenseKey", this.message.payload.key);
  }

  userDistracted() {
    let insightsData = state.getState("insightsData");
    if (insightsData.hasOwnProperty("dailyDistractionList")) {
      let today = new Date();

      let dd = String(today.getDate()).padStart(2, "0");
      let mm = String(today.getMonth() + 1).padStart(2, "0"); //January is 0!
      let yyyy = today.getFullYear();
      today = `${dd}/${mm}/${yyyy}`;

      let data = insightsData.dailyDistractionList;

      if (data.hasOwnProperty(today)) {
        // update the count
        data[today] = data[today] + 1;
      } else {
        // add key to data
        data[today] = 1;
      }

      insightsData.dailyDistractionList = data;
      state.setState("insightsData", insightsData);
    } else {
      insightsData["dailyDistractionList"] = {};
      state.setState("insightsData", insightsData);
    }
  }

  updateInsights() {
    const insights = this.message.payload.insights;
    state.setState("insightsData", insights);
    this.sendResponse({ status: "OK" });
  }

  soundToggle() {
    const soundsOn = state.getState("soundsOn");
    state.setState("soundsOn", !soundsOn);
  }

  breakModeToggle() {
    const disableBlockInBreakMode = state.getState("disableBlockInBreakMode");
    state.setState("disableBlockInBreakMode", !disableBlockInBreakMode);
  }

  notificationToggle() {
    const notification = state.getState("pushNotificationsEnabled");
    state.setState("pushNotificationsEnabled", !notification);
  }

  toggleHardMode() {
    const hardModeState = state.getState("hardMode");
    state.setState("hardMode", !hardModeState);
  }

  toggleAutoBlock() {
    const autoBlockState = state.getState("autoblock");
    state.setState("autoblock", !autoBlockState);
  }

  redirectToBlockPage() {
    const redirectUrl = this.message.payload.redirect;
    chrome.tabs.update(this.sender.tab.id, { url: redirectUrl });
  }

  startAnimation() {
    const animationType = this.message.animationName;
    chrome.runtime.sendMessage({
      action: "startAnimation",
      animationName: animationType,
    });
  }

  startBreakTimer() {}

  requestUnblockURL() {
    const url = this.message.payload.url;
    const blocklist = state.getState("blockedSites");

    if (blocklist.includes(url)) {
      const indexOfURL = blocklist.indexOf(url);
      if (indexOfURL > -1) {
        blocklist.splice(indexOfURL, 1);
        state.setState("blockedSites", blocklist);
      }
      this.sendResponse({ status: "OK" });
    }
  }

  requestAutoBlockDeleteURL() {
    const url = this.message.payload.url;
    let autoblocklist = state.getState("autoBlockedSites");
    autoblocklist.splice(
      autoblocklist.findIndex((a) => a.siteName === url),
      1
    );
    state.setState("autoBlockedSites", autoblocklist);
    this.sendResponse({ status: "OK" });
  }

  requestAutoBlockUrlOff() {
    const currentSite = this.message.payload.url;
    const autoblockList = state.getState("autoBlockedSites");

    autoblockList.map((site) => {
      if (site.siteName === currentSite) {
        site.blocked = false;
      }
    });

    state.setState("autoBlockedSites", autoblockList);
    this.sendResponse({ status: "OK" });
  }

  requestBlockList() {
    const blocklist = state.getState("blockedSites");
    this.sendResponse({ status: "OK", data: blocklist });
  }

  setBreakTimer() {
    const timer = this.message.payload.timer;
    const minutes = parseInt(timer.split(":")[0]);
    const seconds = parseInt(timer.split(":")[1]);
    const totalSeconds = minutes * 60 + seconds;
    state.setState("breakCount", totalSeconds);
  }

  setCountTimer() {
    const timer = this.message.payload.timer;
    const minutes = parseInt(timer.split(":")[0]);
    const seconds = parseInt(timer.split(":")[1]);
    const totalSeconds = minutes * 60 + seconds;
    state.setState("timerCount", totalSeconds);
  }

  getAllState() {
    this.sendResponse({ status: "OK", payload: state.getStates() });
  }

  getActions() {
    return this.actionObject;
  }

  addSiteToBlock() {
    const url = this.message.url;
    const strippedBase = url
      .replace(/^(?:https?:\/\/)?(?:www\.)?/i, "")
      .split("/")[0];

    const blockedSites = state.getState("blockedSites");
    if (!blockedSites.includes(strippedBase)) {
      const newBlockedList = [...blockedSites, strippedBase];
      state.setState("blockedSites", newBlockedList);
    }

    this.sendResponse({ status: "OK" });
  }

  addSiteToAutoblock() {
    // if site is not present in the list, add it
    // if it exists then update the time value
    const response = this.message.payload;
    const url = response.url;
    const durationInMin = parseInt(response.time);
    const autoblockedSites = state.getState("autoBlockedSites");

    const siteExists = autoblockedSites.find((site) => site.siteName === url);
    if (siteExists === undefined) {
      const newSite = {
        siteName: url,
        duration: durationInMin * 60,
        blocked: false,
      };
      const newBlockedList = [...autoblockedSites, newSite];
      state.setState("autoBlockedSites", newBlockedList);
    } else {
      siteExists.duration = durationInMin * 60;
      siteExists.blocked = false;
    }

    // event to re-render the UI
    this.sendResponse({ status: "OK" });
  }

  switchScreen() {
    const screen = this.message.payload.currentScreenState.toString();
    // console.log("screen switched to ->", screen);
    const prevState = state.getState("currentScreenState");
    state.setState("currentScreenState", screen);
    this.sendResponse({ status: "OK", prevState: prevState });
  }

  updateTimer() {
    state.setState("timerCount", this.message.payload.value);
  }

  getTimerState() {
    const timerState = state.getState("isTimerOn");
    const isTimerPaused = state.getState("isPaused");
    this.sendResponse({ status: "OK", payload: { timerState, isTimerPaused } });
  }

  siteVisited() {
    const blockedSites = state.getState("blockedSites");
    const isTimerOn = state.getState("isTimerOn");
    const isBreakTime = state.getState("isBreakTime");
    const disableBlockInBreakMode = state.getState("disableBlockInBreakMode");
    const shouldUnblockInBreak =
      isTimerOn && isBreakTime && disableBlockInBreakMode;
    // if timer is on and its not break time - block site
    // if timer is on and is break time - block site
    // if timer is on and is break time and disableBlockInBreakMode is true - unblock

    // option to turn of block during break mode

    let currentSite = this.message.payload.currentSite;
    currentSite = currentSite
      .replace(/^(?:https?:\/\/)?(?:www\.)?/i, "")
      .split("/")[0];
    // only runs if the disableBlockInBreakMode is false
    if (!shouldUnblockInBreak) {
      if (blockedSites.includes(currentSite) && isTimerOn) {
        chrome.tabs.getSelected(null, function (tab) {
          chrome.tabs.sendMessage(tab.id, {
            action: "blockSite",
            payload: { currentSite: window.location.hostname },
          });
        });
      }
    }
  }

  autoBlockCheck() {
    try {
      const currentSite = stripurl(this.message.payload.currentSite);
      const autoBlockList = state.getState("autoBlockedSites");
      const isAutoBlockOn = state.getState("autoblock");

      if (isAutoBlockOn) {
        const siteMatch = autoBlockList.filter(
          (site) => site.siteName === currentSite
        );
        if (siteMatch.length !== 0 && !siteMatch[0].blocked) {
          const timestampWhenLanded = Date.now();
          activateAutoblock(timestampWhenLanded, siteMatch[0]);
        }
      } else {
        console.log("Autoblock is off");
      }
    } catch (err) {
      console.error(err);
    }
  }

  pauseTimer() {
    state.setState("isPaused", true);
    this.sendResponse({ status: "OK" });
  }

  playTimer() {
    state.setState("isPaused", false);
    this.sendResponse({ status: "OK" });
  }

  startTimer() {
    state.setState("isTimerOn", true);
    this.sendResponse({ status: "OK" });
  }

  stopTimer() {
    state.setState("isTimerOn", false);
    this.sendResponse({ status: "OK" });
  }

  depleteHealth() {
    const healthToRemove = this.message.payload.health;
    const currentHealth = parseInt(state.getState("health"));
    let newHealth = 0;

    if (currentHealth > 0) {
      newHealth = currentHealth - healthToRemove;
      state.setState("health", newHealth);
      chrome.runtime.sendMessage({
        action: "updateHealthMeter",
        health: newHealth,
      });
    } else {
      // regenerate
      const insightsData = state.getState("insightsData");
      // otto regeneration code
      regenerate(currentHealth, insightsData, newHealth);
    }
  }
}

function regenerate(currentHealth, insightsData, newHealth) {
  if (currentHealth === 0) {
    if (insightsData.hasOwnProperty("regenerationCount")) {
      insightsData.regenerationCount =
        parseInt(insightsData.regenerationCount) + 1;
      state.setState("insightsData", insightsData);
    } else {
      insightsData["regenerationCount"] = 0;
      state.setState("insightsData", insightsData);
    }

    newHealth = 100;
    state.setState("health", newHealth);
    chrome.runtime.sendMessage({
      action: "updateHealthMeter",
      health: newHealth,
    });

    let colors = ["blue", "red", "yellow"];
    let item = colors[Math.floor(Math.random() * colors.length)];

    state.setState("currentColor", item);
  }
}

// check if current site exists in autoblock list
// if it does, record time stamp
// request bg script to start the timer that runs every minute
// in the interval callback check if the difference between the initial timestamp and
// the current timestamp is equal to the auto block duration
// if yes, clear the timer and block the site

let allActiveTabs = [];

function activateAutoblock(startTime, siteObject) {
  // if new tabs are created, after autoblock was started
  // this ensures that they are added to the active tabs list
  function autoBlockSetInterval() {
    const then = startTime / 1000;
    const now = Date.now() / 1000;
    const endDuration = siteObject.duration;
    const diff = Math.round(now - then);
    const percent = Math.round((100 * parseInt(diff)) / parseInt(endDuration));
    console.log(
      `[DEBUG]: time : ${diff}; completed: ${percent}%; site: ${siteObject.siteName}`
    );

    if (diff >= endDuration) {
      chrome.alarms.clear(siteObject.siteName, (cleared) => {
        if (cleared) {
          const autoBlockList = state.getState("autoBlockedSites");
          autoBlockList.map((site) => {
            if (site.siteName === siteObject.siteName) {
              site.blocked = true;
            }
          });

          state.setState("autoBlockedSites", autoBlockList);
          chrome.tabs.query({}, (result) => {
            result.map((x) => {
              const url = stripurl(x.url);
              if (url === siteObject.siteName) {
                const blockPageUrl = chrome.runtime.getURL("/blockedUI.html");
                chrome.tabs.update(x.id, { url: blockPageUrl });
              }
            });
          });
        }
      });
    }
  }

  if (!siteObject.blocked) {
    // send request to bg script
    let runEverySecond = 1;
    try {
      chrome.alarms.create(siteObject.siteName, {
        periodInMinutes: runEverySecond,
      });

      chrome.alarms.onAlarm.addListener(autoBlockSetInterval);
    } catch {
      console.log("[ERROR]: failed to create a timer");
    }
  } else {
    console.log("[DEBUG]: site already blocked, no need to run.");
  }
}
