let insightsDataDurationSpent = {};
let insightsWeeklyData = {};

const sortedValues = (data) => {
  return Object.entries(data)
    .sort((a, b) => b[1] - a[1])
    .reduce(
      (_sortedObj, [k, v]) => ({
        ..._sortedObj,
        [k]: v,
      }),
      {}
    );
};

function getTopDurationsData(data, count = 8) {
  const sortedDecending = sortedValues(data);
  const sliced = Object.keys(sortedDecending)
    .slice(0, count)
    .reduce((result, key) => {
      result[key] = sortedDecending[key];

      return result;
    }, {});

  return sliced;
}

function getSummedTime(data) {
  let summedTime = {};
  Object.keys(data).forEach(function (key) {
    let s = 0;
    Object.values(data[key]).map((k) => {
      s += k.seconds;
    });

    summedTime[key] = s;
  });
  return summedTime;
}

function generateDayFormat(date) {
  return date
    .toLocaleDateString("en-US", {
      day: "numeric",
      month: "short",
      year: "numeric",
    })
    .replace(/ /g, "/")
    .replace(",", "");
}

chrome.storage.local.get(["topInsightsData"], function (result) {
  Object.assign(
    insightsDataDurationSpent,
    getTopDurationsData(result.topInsightsData || {})
  );
});

chrome.storage.local.get(["weeklyInsights"], function (result) {
  // console.log("weekly ->", result.weeklyInsights || {});
  Object.assign(insightsWeeklyData, result.weeklyInsights || {});
});

class GlobalTrackingManager {
  /**
   * Bridge between the local storage and the tracker
   * Utility functions to manage trackers
   */
  constructor() {
    this.globalTracker = [];
  }

  addTracker(tracker) {
    if (!this.containsTracker(tracker)) {
      this.globalTracker.push(tracker);
      // console.log(`tracker added for ${stripURL(tracker.url)}`);
    }
  }

  getAllTrackers() {
    return this.globalTracker;
  }

  removeTracker(tracker) {
    this.globalTracker = this.globalTracker.filter((t) => {
      return t.id !== tracker.id;
    });
    // console.log(`tracker removed for ${stripURL(tracker.url)}`);
  }

  containsTracker(obj) {
    let i;
    for (i = 0; i < this.globalTracker.length; i++) {
      if (this.globalTracker[i].id === obj.id) {
        return true;
      }
    }
    return false;
  }
}

class TimeKeeperManager {
  constructor() {
    this.allWebsiteDuration = insightsDataDurationSpent;
    this.weeklyDuration = insightsWeeklyData;
  }

  saveTopUsageInsight(value) {
    try {
      let v = getTopDurationsData(value);
      chrome.storage.local.set({ topInsightsData: v }, () => {
        console.log("saved daily insights...");
      });
    } catch (error) {
      console.log("error -", error);
    }
  }

  saveWeeklyUsageInsight(data) {
    try {
      let summedTime = getSummedTime(data);
      let topSitesWeek = Object.keys(getTopDurationsData(summedTime, 10));
      let newSiteObjData = {};

      Object.keys(data).forEach(function (key) {
        if (topSitesWeek.includes(key)) {
          newSiteObjData[key] = data[key];
        }
      });

      chrome.storage.local.set({ weeklyInsights: newSiteObjData }, () => {
        console.log("saved WeeklyUsageInsight...");
      });
    } catch (error) {
      console.log("error -", error);
    }
  }

  add(baseurl, seconds) {
    const formattedDate = generateDayFormat(new Date());

    let shouldUpdateWeekObject =
      this.weeklyDuration.hasOwnProperty(baseurl) &&
      this.weeklyDuration[baseurl].hasOwnProperty(formattedDate);

    if (shouldUpdateWeekObject) {
      // console.log("object has property: must update");
      // update property with existing data and update the seconds
      this.weeklyDuration[baseurl] = { ...this.weeklyDuration[baseurl] };
      this.weeklyDuration[baseurl][formattedDate].seconds += seconds;
      // console.log("property updated for week -", baseurl);
      this.saveWeeklyUsageInsight(this.weeklyDuration);
    } else {
      // console.log("object has no property: must create");
      // create property if it don't exist
      this.weeklyDuration[baseurl] = { ...this.weeklyDuration[baseurl] };
      const formattedDate = generateDayFormat(new Date());
      this.weeklyDuration[baseurl][formattedDate] = {
        seconds: seconds,
        day: new Date(),
      };
      // console.log("property created for week -", baseurl);
      this.saveWeeklyUsageInsight(this.weeklyDuration);
    }

    if (Object.keys(this.allWebsiteDuration).includes(baseurl)) {
      // console.log("updated -", baseurl);
      this.allWebsiteDuration[baseurl] =
        this.allWebsiteDuration[baseurl] + seconds;
      // save duration data
      this.saveTopUsageInsight(this.allWebsiteDuration);
    } else {
      // console.log("created -", baseurl);
      this.allWebsiteDuration[baseurl] = seconds;
      this.saveTopUsageInsight(this.allWebsiteDuration);
    }
  }
}

const Manager = new GlobalTrackingManager();
const TimeKeeper = new TimeKeeperManager();

class Tracker {
  constructor(id, name, url) {
    this.id = id;
    this.name = name;
    this.url = url;
    this.startTime = new Date();
    this.endTime = null;
    this.addTracker();
  }

  addTracker() {
    Manager.addTracker(this);
  }

  close() {
    this.endTime = new Date();
    let diff = this.endTime - this.startTime;
    let seconds = parseInt(diff / 1000);
    let baseurl = stripURL(this.url);
    // console.log(`You were on ${baseurl} for ${seconds}sec`);

    TimeKeeper.add(baseurl, seconds);
    Manager.removeTracker(this);
  }
}

chrome.tabs.onUpdated.addListener((tabId) => {
  try {
    chrome.tabs.get(tabId, (currentTab) => {
      if (isValidUrl(currentTab.url)) {
        const { title, id, url } = currentTab;
        const t = new Tracker(id, title, url);
      }
    });
  } catch (error) {
    console.log("something went wrong!");
  }
});

chrome.tabs.onRemoved.addListener((tabId) => {
  try {
    Manager.getAllTrackers().map((tracker) => {
      if (tracker.id === tabId) {
        tracker.close();
      }
    });
  } catch (error) {
    console.log("something went wrong!");
  }
});
