console.log(
  "Always with you what cannot be done. Hear you nothing that I say?...You must unlearn what you have learned...Try not. Do. Or do not. There is no try."
);

const state = new AppState();
const blockedURLs = state.getState("blockedSites");
let allRunningTimers = [];

chrome.storage.local.get(["appData", "topInsightsData"], function (result) {
  if (result.appData) {
    // need to compare the state from app data and the local state so that any new properties
    // are added to the state object

    if (state !== undefined) {
      let data = result.appData;
      const currentState = state.getStates();

      // add new state properties when available into the state object from local storage
      Object.keys(currentState).map((sKey) => {
        if (sKey in data) {
        } else {
          data[sKey] = currentState[sKey];
          console.log("[INFO] New property added:", sKey);
        }
      });

      // need to write a function that can delete unused properties

      data.currentScreenState = "startScreen";
      data.isTimerOn = false;
      data.isBreakTime = false;
      data.topSitesData = result.topInsightsData;
      state.setAll(data);
    }
  }
});

chrome.runtime.onInstalled.addListener(function () {
  chrome.browserAction.setBadgeText({ text: "" });
  chrome.browserAction.setBadgeBackgroundColor({ color: "#FB28FF" });
});

function clearTimers() {
  const runningTimersCopy = allRunningTimers;
  allRunningTimers.map((timerId) => {
    const index = allRunningTimers.indexOf(timerId);
    if (index > -1) {
      clearInterval(timerId);
      allRunningTimers.splice(index, 1);
    }
  });
}

chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
  const AB = new ActionBlocks(message, sender, sendResponse);
  const actions = AB.getActions();
  // call function with corresponding action type
  actions[message.action]();
  // to start the timer
  if (message.action === "startTimer") {
    startWorkTimer();
    chrome.runtime.sendMessage({
      action: "startAnimation",
      animationName: "workout",
    });
  }

  // stopping the timer
  if (message.action === "stopTimer") {
    chrome.browserAction.setBadgeText({ text: "" });
    clearTimers();

    chrome.runtime.sendMessage({
      action: "startAnimation",
      animationName: "idle",
    });
  }

  if (message.action === "startBreakTimer") {
    startBreakTimer();
    chrome.runtime.sendMessage({
      action: "startAnimation",
      animationName: "workout",
    });
  }
});

function startWorkTimer() {
  chrome.browserAction.setBadgeBackgroundColor({ color: "#FB28FF" });
  startTimer(state.getState("timerCount"), "workTimer");
}

function startBreakTimer() {
  // break timer
  chrome.browserAction.setBadgeBackgroundColor({ color: "#0D615C" });
  startTimer(state.getState("breakCount"), "breakTimer");
}

// **** UTILS ******

function isValidUrl(str) {
  var pattern = new RegExp(
    "^(https?:\\/\\/)?" + // protocol
      "((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|" + // domain name
      "((\\d{1,3}\\.){3}\\d{1,3}))" + // OR ip (v4) address
      "(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*" + // port and path
      "(\\?[;&a-z\\d%_.~+=-]*)?" + // query string
      "(\\#[-a-z\\d_]*)?$",
    "i"
  ); // fragment locator
  return !!pattern.test(str);
}

function stripURL(url) {
  return url.replace(/^(?:https?:\/\/)?(?:www\.)?/i, "").split("/")[0];
}

function generateBlockURLVariations(urls) {
  let blockedVariations = [];
  urls.map((url) => {
    if (url !== "") {
      const strippedBase = url
        .replace(/^(?:https?:\/\/)?(?:www\.)?/i, "")
        .split("/")[0];

      blockedVariations.push([
        `*://*.${strippedBase}/*`,
        `*://${strippedBase}/*`,
        `*://www.${strippedBase}/*`,
      ]);
    }
  });
  return [].concat.apply([], blockedVariations);
}

function percentage(partialValue, totalValue) {
  return ((100 * partialValue) / totalValue).toFixed(0);
}

function startTimer(duration, timerType) {
  let timer = duration,
    minutes,
    seconds;
  let percent;
  const timerID = setInterval(function () {
    const isTimerOn = state.getState("isTimerOn");
    const isPaused = state.getState("isPaused");

    if (isTimerOn) {
      minutes = parseInt(timer / 60, 10);
      seconds = parseInt(timer % 60, 10);
      minutes = minutes < 10 ? "0" + minutes : minutes;
      seconds = seconds < 10 ? "0" + seconds : seconds;
      percent = percentage(timer, duration);

      // update the timer
      chrome.runtime.sendMessage({
        action: "setPercentage",
        payload: { percentage: percent, count: `${minutes}:${seconds}` },
      });

      if (isPaused) {
        chrome.browserAction.setBadgeText({ text: "PAUSED" });
      } else {
        chrome.browserAction.setBadgeText({ text: minutes + ":" + seconds });
      }

      if (!isPaused) {
        if (--timer < 0) {
          timer = duration;
          // when work timer ends, send request to start break timer
          if (timerType === "workTimer") {
            clearTimers();
            state.setState("isBreakTime", true);

            let myAudio = new Audio(chrome.runtime.getURL("sounds/bell.mp3"));

            if (state.getState("soundsOn")) {
              myAudio.play();
            }

            // updating the UI to reflect the state
            chrome.runtime.sendMessage({
              action: "requestBreakTimer",
              payload: null,
            });

            // add work time to data (total time spent working)
            countTotalWorkTime();

            if (state.getState("pushNotificationsEnabled")) {
              try {
                chrome.notifications.getPermissionLevel((level) => {
                  if (level === "granted") {
                    let icon = chrome.runtime.getURL(
                      "images/icons/icon128.png"
                    );
                    chrome.notifications.create("", {
                      type: "basic",
                      title: "Take a break!",
                      message: "Good job, now go take a break!",
                      iconUrl: icon,
                      priority: 2,
                    });
                  }
                });
              } catch {}
            }

            // starting the timer in the bg
            startBreakTimer();
          }

          // when break timer ends, send request to start work timer
          if (timerType === "breakTimer") {
            clearTimers();
            state.setState("isBreakTime", false);

            let myAudio = new Audio(chrome.runtime.getURL("sounds/bell.mp3"));

            if (state.getState("soundsOn")) {
              myAudio.play();
            }

            // updating the UI to reflect the state
            chrome.runtime.sendMessage({
              action: "requestWorkTimer",
              payload: null,
            });

            if (state.getState("pushNotificationsEnabled")) {
              try {
                chrome.notifications.getPermissionLevel((level) => {
                  if (level === "granted") {
                    let icon = chrome.runtime.getURL(
                      "images/icons/icon128.png"
                    );
                    chrome.notifications.create("", {
                      type: "basic",
                      title: "Break's up",
                      message: "Time to get back to work",
                      iconUrl: icon,
                      priority: 2,
                    });
                  }
                });
              } catch {}
            }

            // starting the timer in the bg
            startWorkTimer();
          }
        }
      }
    } else {
      timer = duration;
    }
  }, 1000);

  allRunningTimers.push(timerID);
}

function countTotalWorkTime() {
  let insightsData = state.getState("insightsData");
  let workTimer = state.getState("timerCount");
  if (insightsData.hasOwnProperty("totalWorkTime")) {
    insightsData.totalWorkTime =
      parseInt(insightsData.totalWorkTime) + workTimer;
    state.setState("insightsData", insightsData);
  } else {
    insightsData.totalWorkTime = 0;
    state.setState("insightsData", insightsData);
  }

  const currentHealth = parseInt(state.getState("health"));
  if (currentHealth < 100) {
    let newHealth = currentHealth + 5;
    state.setState("health", newHealth);
  }
}
